@@ -19,7 +19,9 @@ definitions it links to.  If not, report the bug!
 # https://tools.ietf.org/html/rfc3696
 
 import string
+from functools import lru_cache
 from importlib import resources
+from typing import Optional
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument
@@ -30,25 +32,24 @@ URL_SAFE_CHARACTERS = frozenset(string.ascii_letters + string.digits + "$-_.+!*'
 FRAGMENT_SAFE_CHARACTERS = URL_SAFE_CHARACTERS | {"?", "/"}
 
 
-# This file is sourced from http://data.iana.org/TLD/tlds-alpha-by-domain.txt
-# The file contains additional information about the date that it was last updated.
-try:  # pragma: no cover
+@lru_cache(maxsize=1)
+def get_top_level_domains() -> tuple[str, ...]:
+    # This file is sourced from http://data.iana.org/TLD/tlds-alpha-by-domain.txt
+    # The file contains additional information about the date that it was last updated.
     traversable = resources.files("hypothesis.vendor") / "tlds-alpha-by-domain.txt"
     _comment, *_tlds = traversable.read_text(encoding="utf-8").splitlines()
-except (AttributeError, ValueError):  # pragma: no cover  # .files() was added in 3.9
-    _comment, *_tlds = resources.read_text(
-        "hypothesis.vendor", "tlds-alpha-by-domain.txt", encoding="utf-8"
-    ).splitlines()
-assert _comment.startswith("#")
+    assert _comment.startswith("#")
 
-# Remove special-use domain names from the list. For more discussion
-# see https://github.com/HypothesisWorks/hypothesis/pull/3572
-TOP_LEVEL_DOMAINS = ["COM", *sorted((d for d in _tlds if d != "ARPA"), key=len)]
+    # Remove special-use domain names from the list. For more discussion
+    # see https://github.com/HypothesisWorks/hypothesis/pull/3572
+    return ("COM", *sorted((d for d in _tlds if d != "ARPA"), key=len))
 
 
 class DomainNameStrategy(st.SearchStrategy):
     @staticmethod
-    def clean_inputs(minimum, maximum, value, variable_name):
+    def clean_inputs(
+        minimum: int, maximum: int, value: Optional[int], variable_name: str
+    ) -> int:
         if value is None:
             value = maximum
         elif not isinstance(value, int):
@@ -61,7 +62,9 @@ class DomainNameStrategy(st.SearchStrategy):
             )
         return value
 
-    def __init__(self, max_length=None, max_element_length=None):
+    def __init__(
+        self, max_length: Optional[int] = None, max_element_length: Optional[int] = None
+    ) -> None:
         """
         A strategy for :rfc:`1035` fully qualified domain names.
 
@@ -101,7 +104,7 @@ class DomainNameStrategy(st.SearchStrategy):
         # prevent us from generating at least a 1 character subdomain.
         # 3 - Randomize the TLD between upper and lower case characters.
         domain = data.draw(
-            st.sampled_from(TOP_LEVEL_DOMAINS)
+            st.sampled_from(get_top_level_domains())
             .filter(lambda tld: len(tld) + 2 <= self.max_length)
             .flatmap(
                 lambda tld: st.tuples(
@@ -146,9 +149,11 @@ def domains(
 _url_fragments_strategy = (
     st.lists(
         st.builds(
-            lambda char, encode: f"%{ord(char):02X}"
-            if (encode or char not in FRAGMENT_SAFE_CHARACTERS)
-            else char,
+            lambda char, encode: (
+                f"%{ord(char):02X}"
+                if (encode or char not in FRAGMENT_SAFE_CHARACTERS)
+                else char
+            ),
             st.characters(min_codepoint=0, max_codepoint=255),
             st.booleans(),
         ),
@@ -161,13 +166,18 @@ _url_fragments_strategy = (
 
 @defines_strategy(force_reusable_values=True)
 def urls() -> st.SearchStrategy[str]:
-    """A strategy for :rfc:`3986`, generating http/https URLs."""
+    """A strategy for :rfc:`3986`, generating http/https URLs.
 
-    def url_encode(s):
+    The generated URLs could, at least in theory, be passed to an HTTP client
+    and fetched.
+
+    """
+
+    def url_encode(s: str) -> str:
         return "".join(c if c in URL_SAFE_CHARACTERS else "%%%02X" % ord(c) for c in s)
 
     schemes = st.sampled_from(["http", "https"])
-    ports = st.integers(min_value=0, max_value=2**16 - 1).map(":{}".format)
+    ports = st.integers(min_value=1, max_value=2**16 - 1).map(":{}".format)
     paths = st.lists(st.text(string.printable).map(url_encode)).map("/".join)
 
     return st.builds(
